/*
 * **********************************************************************
 *
 *   Annotate.c ==> routines that implement annotation of list items.
 *
 * ----------------------------------------------------------------------
 * Copyright (c) 1995 by Mike Jipping and Hope College
 *
 * Permission is granted to copy and distribute this file in modified or
 * unmodified form, for noncommercial use, provided (a) this copyright notice
 * is preserved, (b) no attempt is made to restrict redistribution of this
 * file, and (c) this file is not distributed as part of any collection whose
 * redistribution is restricted by a compilation copyright.
 * ----------------------------------------------------------------------
 */

#include "globaldefs.h"
#include <unistd.h>

char id[30];
aprint_aprint_base_objects	*Aprint_aprint_base;

/*
 * **********************************************************************
 * This routine generates a new annotation ID.  It's just the date from
 * the "epoch" in seconds.
 */
char *new_id ()
{
#ifdef linux
   struct timezone tz;
#endif
   struct timeval tv;
   char *id;

#ifdef linux
   gettimeofday(&tv, &tz);
#else
   gettimeofday(&tv);
#endif
   id = (char *)malloc(sizeof(char)*30);
   sprintf(id, "%d%d", tv.tv_sec, tv.tv_usec);

   return id;
}
/*
 * **********************************************************************
 * Create the annotation directory with the correct file mode.
 */
void create_annotation_tree()
{
	mode_t mode = 493;

	if (access(annotation_dir, F_OK)) {
		mkdir(annotation_dir, mode);
	}
}

/*
 * **********************************************************************
 * This routine starts the edit of the annotation.  It makes sure that
 * the directory structure exists, then calls the editor.
 */
void open_annotation (annot_name)
char *annot_name;
{
	FILE *annfd;
	char path[256], cmd[300];

	create_annotation_tree();
	sprintf(path, "%s/%s", annotation_dir, annot_name);
	annfd = fopen(path, "a+");
	if (annfd == NULL) {
		return;
	}
	fclose(annfd);

	sprintf(cmd, "%s %s", annotation_editor, path);
	system(cmd);
}

/*
 * **********************************************************************
 * Notify callback function for the annotation edit menu entry.
 */
void edit_annotation(item, event)
Panel_item item;
Event *event;
{
	xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, 0);
	if (strlen(id) == 0) strcpy(id, new_id());
	open_annotation(id);
}

/*
 * **********************************************************************
 * Begin the print process for the "print annotation" function.
 */
void print_selection_annotation(de)
struct day_entry *de;
{
	char cmd[1024];

#ifdef SVR4		
	sprintf(cmd, "lp -c -s -t \"Annotation for '%s'\" -d ", de->text);
#else
	sprintf(cmd, "lpr -t \"Annotation for '%s'\" -P", de->text);
#endif
	xv_set(Aprint_aprint_base->aprint_printer, PANEL_VALUE, cmd, 0);
	xv_set(Aprint_aprint_base->aprint_print, PANEL_CLIENT_DATA, de, 0);
	xv_set(Aprint_aprint_base->aprint_base, XV_SHOW, TRUE, 0);
}

/*
 * **********************************************************************
 * Annotate a selected item from the list.  Note the list redisplay
 * because the glyph will change for a new annotation.
 */
void annotate_selection(de)
struct day_entry *de;
{
	if (strlen(de->entry_id) == 0) strcpy(de->entry_id, new_id());
	open_annotation(de->entry_id);
	display_list(curr_month, curr_day, curr_year);
	changed = TRUE;
	refresh_db(FALSE);
}
 
/*
 * **********************************************************************
 * DEannotate a selected item from the list -- find and unlink an
 * annotation file, if necessary.  Note the list redisplay because the
 * glyph will change for a new annotation.
 */
void deannotate_selection(de)
struct day_entry *de;
{
	if (strlen(de->entry_id) == 0) return;
	unlink(de->entry_id);
	de->entry_id[0] = '\0';
	display_list(curr_month, curr_day, curr_year);
	changed = TRUE;
	refresh_db(FALSE);
}

/*
 * **********************************************************************
 * Close the annotation print window and print the annotation on the
 * selected printer.
 */
void
aprint_printit(Panel_item item, Event *event)
{
	struct day_entry *de;
	char path[256], tmpcmd[256], printercmd[1024];

	de = (struct day_entry *)xv_get(Aprint_aprint_base->aprint_print, PANEL_CLIENT_DATA);
	strcpy(tmpcmd, (char *)xv_get(Aprint_aprint_base->aprint_printer, PANEL_VALUE));
	if ((int)strlen(tmpcmd) > 0) {
		sprintf(path, "%s/%s", annotation_dir, de->entry_id);
		sprintf(printercmd, "%s %s", tmpcmd, path);
		system(printercmd);
	} 
}

/*
 * **********************************************************************
 * Setup the annotation print window to have a certain printer.
 */
void initialize_aprint()
{
	Aprint_aprint_base = aprint_aprint_base_objects_initialize(NULL, tdlist);

	if ((int)strlen(default_printer) > 0) {
		xv_set(Aprint_aprint_base->aprint_printer, PANEL_VALUE, default_printer, 0);
	} else {
		xv_set(Aprint_aprint_base->aprint_printer, PANEL_VALUE, getenv("PRINTER"), 0);
	}
}
